@@ -29,6 +29,8 @@
 #include "i2c-dev.h"
 #include "setpanel.h"
 
+#include "soundctrl.h"
+
 BOOL bI2CInitialised = FALSE;
 
 static struct option longopts[] = {
@@ -456,11 +458,12 @@ int main(int argc, char *argv[])
 
   BOOL bUpdateTemp = FALSE;
   BOOL bUpdateTime = FALSE;
+  BOOL bUpdateVolume = FALSE;
 
   const struct tm * nice_time;
   time_t nasty_time;
   char hour, minute;
-  int volume;
+  char volume;
 
   if (argc == 1)
   {
@@ -619,9 +622,9 @@ int main(int argc, char *argv[])
                 }
 		printf("\n");
 		if (ch == 'D')
-                digimatrix_DisplayNumbers(file, digits, led, pp, dots, 1);
+                digimatrix_DisplayNumbers(file, digits, led, pp, dots, (BOOL)1);
 		else
-                digimatrix_DisplayNumbers(file, digits, led, pp, dots, 0);
+                digimatrix_DisplayNumbers(file, digits, led, pp, dots, (BOOL)0);
 	    }
             break;
             
@@ -643,6 +646,12 @@ int main(int argc, char *argv[])
             
         case 'l':
             printf("display volume control. \nVolume Change: ");
+            //digimatrix_write_byte(file, 0x24);
+            //volume = digimatrix_read_byte(file);
+            //printf("%d\n", volume);
+	    bUpdateVolume = TRUE;
+	    break;
+
 	case 'L':
             digimatrix_write_byte(file, 0x24);
             volume = digimatrix_read_byte(file);
@@ -702,22 +711,23 @@ int main(int argc, char *argv[])
         }
     }
 
-    if (bUpdateTemp)
+    if (bUpdateTemp || bUpdateTime || bUpdateVolume)
     {
         unsigned int temp_counter = 0;
+        time_t current_time;
+        struct tm *time_date;
+        LED led = LED_NONE;
+        PP pp = PP_NONE;
+        static DOTS dots = DOTS_NONE;
+        char ascii_time[5];
+	soundCtrl SoundControl("/dev/mixer");
 
         while (1)
         {
-            if (bUpdateTemp && bUpdateTime)
+            if (bUpdateTemp || bUpdateTime)
             {
                 if (temp_counter >= (TEMPERATURE_UPDATE_INTERVAL_MS / VOLUME_UPDATE_INTERVAL_MS))
                 {
-		    time_t current_time;
-		    struct tm *time_date;
-	            LED led = LED_NONE;
-		    PP pp = PP_NONE;
-		    static DOTS dots = DOTS_NONE;
-		    char ascii_time[5];
 
                     // Make the dots flash. 
                     if (dots == DOTS_NONE)
@@ -732,7 +742,7 @@ int main(int argc, char *argv[])
 		    strftime(ascii_time, 5, "%H%M", time_date);
 		    
 		    sprintf(digits, "%02X%s", digimatrix_GetTemp(), ascii_time);
-                    digimatrix_DisplayNumbers(file, digits, led, pp, dots, 0);
+                    digimatrix_DisplayNumbers(file, digits, led, pp, dots, (BOOL)0);
                 }
             }
             else if (bUpdateTemp)
@@ -745,6 +755,38 @@ int main(int argc, char *argv[])
                 }
             }
 
+	    if (bUpdateVolume)
+	    {
+                digimatrix_write_byte(file, 0x24);
+                volume = digimatrix_read_byte(file);
+		//printf("Volume: %d\n\r", volume);
+		
+		if (volume != -120 && volume != 0)
+		{
+		    for (temp_counter=12; temp_counter > 0; temp_counter--)
+		    {
+		        if (volume > 0)
+                        {
+		            SoundControl.volumeUp(volume);
+		        }
+		        else if (volume < 0)
+		        {
+    		            SoundControl.volumeDown(volume);
+	                }
+			
+			dots = DOTS_NONE;
+		        sprintf(digits, "%02d    ", SoundControl.getVolume() / 258);
+		        digimatrix_DisplayNumbers(file, digits, led, pp, dots, (BOOL)0);
+                        
+			usleep(VOLUME_UPDATE_INTERVAL_MS * 1000); // Delay of 500ms
+                        digimatrix_write_byte(file, 0x24);
+                        volume = digimatrix_read_byte(file);
+		    }
+		}
+		
+		
+	    }
+
             usleep(VOLUME_UPDATE_INTERVAL_MS * 1000); // Delay of 500ms
             temp_counter++;
         }
